@@ -1,5 +1,6 @@
 """Stabilize unstable.cfg's development eggs into tag checkouts for stable.cfg
 """
+import difflib
 import logging
 import sys
 import os
@@ -90,13 +91,32 @@ def determine_tags(directories):
     return results
 
 
+def url_list():
+    """Return version lines"""
+    stable_lines = open('stable.cfg').read().split('\n')
+    part = buildoututils.extract_parts(stable_lines, partname=PARTNAME)
+    if not part:
+        return []
+    (start,
+     end,
+     specs,
+     url_section) = buildoututils.extract_option(
+        stable_lines,
+        'urls',
+        startline=part['start'],
+        endline=part['end'])
+    # http://somethinglong/tags/xyz something
+    interesting = [spec.split('tags/')[1] for spec in specs]
+    logger.debug("Version lines: %s", interesting)
+    return interesting
+
+
 def remove_old_part():
     """If PARTNAME already exists, remove it"""
     stable_lines = open('stable.cfg').read().split('\n')
     old_part = buildoututils.extract_parts(stable_lines, partname=PARTNAME)
     if old_part:
         del stable_lines[old_part['start']:old_part['end']]
-
     contents = '\n'.join(stable_lines)
     open('stable.cfg', 'w').write(contents)
     logger.debug("New stable.cfg written: old part removed.")
@@ -134,6 +154,35 @@ def check_stable():
     logger.info("Make sure ${%s:eggs} is actually included.", PARTNAME)
 
 
+def insert_msg_into_history(msg):
+    filename = utils.history_file()
+    if not filename:
+        return
+    lines = open(filename).read().splitlines()
+    headings = utils.extract_headings_from_history(lines)
+    if not headings:
+        return
+    # Hardcoded zest assumptions.
+    target_line = headings[0]['line'] + 3
+    if len(lines) < target_line:
+        return
+    if 'nothing yet' in lines[target_line].lower():
+        del lines[target_line]
+    # Msg formatting:
+    msg = msg[:]
+    msg = ['  %s' % line for line in msg]
+    msg += ['']
+    firstline = msg[0]
+    firstline = '-' + firstline[1:]
+    msg[0] = firstline
+    lines[target_line:target_line] = msg
+    contents = '\n'.join(lines)
+    # Add newline at end of file:
+    contents += '\n'
+    open(filename, 'w').write(contents)
+    logger.debug("Written change to history file")
+
+
 def main():
     logging.basicConfig(level=utils.loglevel(),
                         format="%(levelname)s: %(message)s")
@@ -141,14 +190,18 @@ def main():
     directories = development_eggs()
     if not directories:
         sys.exit()
+    old_situation = url_list()
     tags = determine_tags(directories)
     remove_old_part()
     add_new_part(tags)
-
+    new_situation = url_list()
+    diff = list(difflib.ndiff(old_situation, new_situation))
+    logger.debug("Diff: %s", diff)
     check_stable()
 
     msg = ["Stabilized buildout to most recent svn tags of our packages:"]
-    msg += ['* %s' % tag for tag in tags]
+    msg += diff
+    insert_msg_into_history(msg)
     msg = '\n'.join(msg)
-    # TODO: add msg to history file
+    
     utils.show_diff_offer_commit(msg)
